#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>
#include <fstream>

using namespace std;

int get_spin_spin_int(int *states, int L){
  //Calculate the total spin spin interaction
  int e=0;

  for(int i=0; i<L; i++){
    int l=i;
    int r;
  
    //Periodic boundary conditions
    r=i+1;
    if(r==L) r=0;
    

    int spin_l=states[l];
    int spin_r=states[r];

    e=e-spin_l*spin_r;
  }    

  return e;
}

int get_magnet_int(int *states, int l){
  //Calculate the magnetisation
  int m=0;

  for(int i=0; i<l; i++){
    int spin=states[i];

    m=m+spin;
  }
  return m;
}

int get_one_spin_spin_int(int *states, int L, int index){
  //Calculate the one spin-spin interaction for one spin
  int l,r;
 
  //Find left and right neigbour regarding periodic boundary conditions
  l = index-1;
  if(l==-1) l=L-1;
  
  r = index+1;
  if(r==L) r=0;


  return -states[l]*states[index]-states[r]*states[index];
}

bool accept_metro(double deltaE, double beta){
  //accept or reject spinflip with metropolis
  double zu=0;
  bool accept=0;
  if(deltaE<=0){
    accept=1;
  }	    
  else{
    zu=double(rand())/double(RAND_MAX);
    if(zu<exp(-beta*deltaE)){
      accept=1;  
    }
  }
  return accept;
}

bool accept_glauber(double deltaE, double beta){
  //accept or reject spinflip with glauber
  double zu=0;
  bool accept=0;
  zu=double(rand())/double(RAND_MAX);

  if(zu<1./(1.+exp(beta*deltaE))){
    accept=1;
  }	    
  else{}
  return accept;
}

int main(){
  // variable declaration

  //System
  int L=20;
  int states[L];
  double E;
  double M;
  double M_s;
  double h=0.;
  double beta=1;

  //variables for steps
  double deltaE=0;
  int p_index=0;


  //variables for average calculation
  double av_E=0; 
  double av_E2=0; 
  double av_M =0.; 
  double av_M2 =0.; 


  //variables for timeseries
  int int_rel=200;
  double relax_time[int_rel];
  int count_relax[int_rel];

  //variables for backcheck
  double lambda1;
  double lambda2;
  double Z;
  double dZ;
  double ddZ;

  //set arrays to zero
  for(int i=0; i<int_rel; i++){
    count_relax[i]=0;
    relax_time[i]=0;
  } 
  for(int i=0;i<L;i++) states[i]=1;

  //Open file for MC data
  ostringstream oss;
  oss << "Out_glauber_" << L << ".dat";
  //oss << "Out_metro_" << L << ".dat";  
  fstream file(oss.str().c_str(), ios::out);

  //Open file for timeseries
  oss.str("");  
  oss << "Relax_m_" << L << ".dat";  
  fstream file2(oss.str().c_str(), ios::out);
  




  //set initial energy
  E=get_spin_spin_int(states,L)+h*get_magnet_int(states,L);
  M_s=get_magnet_int(states,L);


  //interavalls for temperature measuments
  double beta_l=0;
  double beta_r=2;
  int res = 100;
  double bin=(beta_r-beta_l)/res;


  ///////////////////////////////////////
  /////// Main temperature loop
  //////////////////////////////////////


  for(int l=0; l<res; l++){
    // set beta
    beta=beta_r-bin*l;

    // set Spins to zero for relaxation measurment
    //for(int i=0;i<L;i++) states[i]=1;
    //E=get_spin_spin_int(states,L)+h*get_magnet_int(states,L);
    //M_s=get_magnet_int(states,L);

    // zero averages
    av_E=0; 
    av_E2=0; 
    av_M =0.; 
    av_M2 =0.; 

    ////////////////////////////////////////
    //////MC main loop
    ///////////////////////////////////////


    for(int k=0; k<8e5; k++){
      //do one sweep
      for(int j=0; j<L; j++){
        //pick spin
        p_index=double(rand())/double(RAND_MAX)*L;
       
        //calc delta E
        deltaE=-2*get_one_spin_spin_int(states,L,p_index)-h*states[p_index]*2;

        //accept/reject
        //if(accept_metro(deltaE,beta)) 
        if(accept_glauber(deltaE,beta)) 
         {
           states[p_index]*=-1;
           E+=deltaE;
           M_s+=2*states[p_index];
         }

     }
     // Do measurments

     // measure relaxation of M
     if(k<int_rel) {
      relax_time[k]=(relax_time[k]*count_relax[k]+fabs(M_s)/L)/(count_relax[k]+1);
      count_relax[k]++;
     }
 
     //measure M,E,Esqared,Msquared
     M=fabs(M_s);
     av_E=(av_E*k+E)/(k+1);
     av_E2=(av_E2*k+E*E)/(k+1);
     av_M=(av_M*k+M)/(k+1);
     av_M2=(av_M2*k+M*M)/(k+1);

 }

 // calculations for backcheck
 lambda1=sinh(beta);
 lambda2=cosh(beta);

 Z=(pow(lambda1,L)+pow(lambda2,L));
 dZ=(L*pow(lambda1,L-1)*lambda2+L*pow(lambda2,L-1)*lambda1);
 ddZ=((L*L-L)*pow(lambda1,L-2)*lambda2*lambda2+(L*L-L)*pow(lambda2,L-2)*lambda1*lambda1+L*Z);
  

 file << beta << "   "
      << av_E/L << "  " 
      << -dZ/Z/L << "  " 
      << beta*beta*(av_E2-av_E*av_E)/(L) << "  " 
      << beta*beta*(-1./(Z*Z)*dZ*dZ+ddZ/Z)/(L) << "  " 
      << av_M/L << "   "
      << beta*(av_M2-av_M*av_M)/(L) << "  "  
      << endl;
 }

 // printout mean relaxation curve
 for(int i=0; i<int_rel; i++){
  file2 << i << "  " << relax_time[i] << endl;
 }

}
